@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2020 Ericsson Telecom AB
+ * Copyright (c) 2000-2021 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v2.0
  * which accompanies this distribution, and is available at
@@ -32,6 +32,7 @@
 #include <sstream>
 
 #include "../common/dbgnew.hh"
+#include "../common/version.h"
 #include "AST.hh"
 #include "asn1/AST_asn1.hh"
 #include "Identifier.hh"
@@ -1319,7 +1320,7 @@ namespace Common {
     control_ns_prefix(p_mt == MOD_ASN ? mcopystr("asn1") : NULL),
     // only ASN.1 modules have default control namespace (X.693 amd1, 16.9)
     used_namespaces(), type_conv_v(), product_number(NULL),
-    suffix(0), release(UINT_MAX), patch(UINT_MAX), build(UINT_MAX), extra(NULL), legacy_version(TUNKNOWN)
+    suffix(0), release(UINT_MAX), patch(UINT_MAX), build(UINT_MAX), extra(NULL), version_type(UNKNOWN)
   {
     if(!p_modid)
       FATAL_ERROR("NULL parameter: Common::Module::Module()");
@@ -1481,7 +1482,7 @@ namespace Common {
 
     if (release <= 999999 && patch < 30 && build < 100) {
       char *product_identifier =
-        get_product_identifier(product_number, suffix, release, patch, build, extra, legacy_version);
+        get_product_identifier(product_number, suffix, release, patch, build, extra, version_type);
       fprintf(stderr, " %s", product_identifier);
       Free(product_identifier);
     }
@@ -1491,7 +1492,7 @@ namespace Common {
 
   char* Module::get_product_identifier(const char* product_number,
         const unsigned int suffix, unsigned int release, unsigned int patch,
-        unsigned int build, const char* extra, tribool legacy)
+        unsigned int build, const char* extra, enum version_t version_type)
   {
     expstring_t ret_val = memptystr();
     if ( product_number == NULL
@@ -1503,11 +1504,15 @@ namespace Common {
       return ret_val;
     }
     if (product_number != NULL) {
-      if (legacy != TTRUE && suffix != 0) {
+      if (version_type == DOT_SEPARATED) {
+    	ret_val = mputprintf(ret_val, "%d.%d.%d", suffix, release, patch);
+    	return ret_val;
+      }
+      if (version_type == LEGACY_CAX && suffix != 0) {
         ret_val = mputprintf(ret_val, "%d/", suffix);
       }
       ret_val = mputstr(ret_val, product_number);
-      if (legacy == TTRUE && suffix != 0) {
+      if (version_type == LEGACY_CRL && suffix != 0) {
         ret_val = mputprintf(ret_val, "/%d", suffix);
       }
       ret_val = mputc(ret_val, ' ');
@@ -1828,6 +1833,8 @@ namespace Common {
       return "template";
     case A_VAR:
       return "variable";
+    case A_EXCEPTION:
+      return "exception";
     case A_VAR_TEMPLATE:
       return "template variable";
     case A_TIMER:
@@ -1884,6 +1891,7 @@ namespace Common {
     case A_PAR_TEMPL_INOUT:
     case A_PAR_TIMER:
     case A_PAR_PORT:
+    case A_EXCEPTION:
       // parameter is identified using its id
       ret_val += id->get_dispname();
       break;
