@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2020 Ericsson Telecom AB
+ * Copyright (c) 2000-2021 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v2.0
  * which accompanies this distribution, and is available at
@@ -35,9 +35,9 @@ void def_encdec(const char *p_classname,
   def=mputstr
     (def,
      "void encode(const TTCN_Typedescriptor_t&, TTCN_Buffer&,"
-     " TTCN_EncDec::coding_t, ...) const;\n"
+     " int, ...) const;\n"
      "void decode(const TTCN_Typedescriptor_t&, TTCN_Buffer&,"
-     " TTCN_EncDec::coding_t, ...);\n");
+     " int, ...);\n");
   if(ber)
     def=mputstr(def,
      "ASN_BER_TLV_t* BER_encode_TLV(const TTCN_Typedescriptor_t& p_td,"
@@ -93,7 +93,7 @@ void def_encdec(const char *p_classname,
 
   src=mputprintf(src,
      "void %s::encode(const TTCN_Typedescriptor_t& p_td,"
-     " TTCN_Buffer& p_buf, TTCN_EncDec::coding_t p_coding, ...) const\n"
+     " TTCN_Buffer& p_buf, int p_coding, ...) const\n"
      "{\n"
      "  va_list pvar;\n"
      "  va_start(pvar, p_coding);\n"
@@ -173,7 +173,7 @@ void def_encdec(const char *p_classname,
      "// written by %s in " __FILE__ " at %d\n"
 #endif
      "void %s::decode(const TTCN_Typedescriptor_t& p_td,"
-     " TTCN_Buffer& p_buf, TTCN_EncDec::coding_t p_coding, ...)\n"
+     " TTCN_Buffer& p_buf, int p_coding, ...)\n"
      "{\n"
      "  va_list pvar;\n"
      "  va_start(pvar, p_coding);\n"
