@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2020 Ericsson Telecom AB
+ * Copyright (c) 2000-2021 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v2.0
  * which accompanies this distribution, and is available at
@@ -780,11 +780,11 @@ void defEnumClass(const enum_def *edef, output_struct *output)
       "      for (rd_ok = p_reader.Read(); rd_ok == 1; rd_ok = p_reader.Read()) {\n"
       "        type = p_reader.NodeType();\n"
       "        if (XML_READER_TYPE_END_ELEMENT == type) {\n"
-      "          rd_ok = p_reader.Read();\n"
+      "          p_reader.Read();\n"
       "          break;\n"
       "        }\n"
       "      }\n"
-      "    else rd_ok = p_reader.Read();\n"
+      "    else p_reader.Read();\n"
       "  }\n"
       "  if (e_xer && (p_flavor & EXIT_ON_ERROR) && %s == enum_value) clean_up();\n" // set to unbound if decoding failed
       "  int decoded_length = 0;\n"
@@ -838,11 +838,16 @@ void defEnumClass(const enum_def *edef, output_struct *output)
       "  size_t value_len = 0;\n"
       "  boolean error = FALSE;\n"
       "  size_t dec_len = 0;\n"
-      "  boolean use_default = p_td.json->default_value && 0 == p_tok.get_buffer_length();\n"
-      "  if (use_default) {\n"
+      "  boolean use_default = FALSE;\n"
+      "  if (p_td.json->default_value.type == JD_STANDARD && 0 == p_tok.get_buffer_length()) {\n"
+      "    *this = *static_cast<const %s*>(p_td.json->default_value.val);\n"
+      "    return dec_len;\n"
+      "  }\n"
+      "  if (p_td.json->default_value.type == JD_LEGACY && 0 == p_tok.get_buffer_length()) {\n"
       // No JSON data in the buffer -> use default value
-      "    value = const_cast<char*>(p_td.json->default_value);\n"
+      "    value = const_cast<char*>(p_td.json->default_value.str);\n"
       "    value_len = strlen(value);\n"
+      "    use_default = TRUE;\n"
       "  } else {\n"
       "    dec_len = p_tok.get_next_token(&token, &value, &value_len);\n"
       "  }\n"
@@ -902,7 +907,7 @@ void defEnumClass(const enum_def *edef, output_struct *output)
       "  }\n"
       "  return (int)dec_len;\n"
       "}\n\n"
-      , name, edef->elements[0].name, enum_type, unknown_value, enum_type
+      , name, name, edef->elements[0].name, enum_type, unknown_value, enum_type
       , unbound_value, unbound_value);
   }
   
@@ -999,7 +1004,12 @@ void defEnumTemplate(const enum_def *edef, output_struct *output)
     "unsigned int n_values;\n"
     "%s_template *list_value;\n"
     "} value_list;\n"
-    "};\n\n", name, enum_type, name);
+    "struct {\n"
+    "%s_template* precondition;\n"
+    "%s_template* implied_template;\n"
+    "} implication_;\n"
+    "dynmatch_struct<%s>* dyn_match;\n"
+    "};\n\n", name, enum_type, name, name, name, name);
 
   /* private members */
   def = mputprintf(def, "void copy_template(const %s_template& "
@@ -1019,6 +1029,7 @@ void defEnumTemplate(const enum_def *edef, output_struct *output)
     "break;\n"
     "case VALUE_LIST:\n"
     "case COMPLEMENTED_LIST:\n"
+    "case CONJUNCTION_MATCH:\n"
     "value_list.n_values = other_value.value_list.n_values;\n"
     "value_list.list_value = new %s_template[value_list.n_values];\n"
     "for (unsigned int list_count = 0; list_count < value_list.n_values; "
@@ -1026,11 +1037,19 @@ void defEnumTemplate(const enum_def *edef, output_struct *output)
     "value_list.list_value[list_count].copy_template("
     "other_value.value_list.list_value[list_count]);\n"
     "break;\n"
+    "case IMPLICATION_MATCH:\n"
+    "implication_.precondition = new %s_template(*other_value.implication_.precondition);\n"
+    "implication_.implied_template = new %s_template(*other_value.implication_.implied_template);\n"
+    "break;\n"
+    "case DYNAMIC_MATCH:\n"
+    "dyn_match = other_value.dyn_match;\n"
+    "dyn_match->ref_count++;\n"
+    "break;\n"
     "default:\n"
     "TTCN_error(\"Copying an uninitialized/unsupported template of "
     "enumerated type %s.\");\n"
     "}\n"
-    "}\n\n", name, name, name, dispname);
+    "}\n\n", name, name, name, name, name, dispname);
 
   def = mputstr(def, "\npublic:\n");
 
@@ -1098,6 +1117,25 @@ void defEnumTemplate(const enum_def *edef, output_struct *output)
     "}\n"
     "}\n\n", name, name, name, enum_type, name, dispname);
 
+  def = mputprintf(def, "%s_template(%s_template* p_precondition, "
+    "%s_template* p_implied_template);\n", name, name, name);
+  src = mputprintf(src, "%s_template::%s_template(%s_template* p_precondition, "
+    "%s_template* p_implied_template)\n"
+    " : Base_Template(IMPLICATION_MATCH)\n"
+    "{\n"
+    "implication_.precondition = p_precondition;\n"
+    "implication_.implied_template = p_implied_template;\n"
+    "}\n\n", name, name, name, name);
+
+  def = mputprintf(def, "%s_template(Dynamic_Match_Interface<%s>* p_dyn_match);\n", name, name);
+  src = mputprintf(src, "%s_template::%s_template(Dynamic_Match_Interface<%s>* p_dyn_match)\n"
+    " : Base_Template(DYNAMIC_MATCH)\n"
+    "{\n"
+    "dyn_match = new dynmatch_struct<%s>;\n"
+    "dyn_match->ptr = p_dyn_match;\n"
+    "dyn_match->ref_count = 1;\n"
+    "}\n\n", name, name, name, name);
+
   def = mputprintf(def, "%s_template(const %s_template& other_value);\n",
                    name, name);
   src = mputprintf(src,
@@ -1138,9 +1176,26 @@ void defEnumTemplate(const enum_def *edef, output_struct *output)
   src = mputprintf(src,
     "void %s_template::clean_up()\n"
     "{\n"
-    "if (template_selection == VALUE_LIST || "
-    "template_selection == COMPLEMENTED_LIST) "
+    "switch (template_selection) {\n"
+    "case VALUE_LIST:\n"
+    "case COMPLEMENTED_LIST:\n"
+    "case CONJUNCTION_MATCH:\n"
     "delete [] value_list.list_value;\n"
+    "break;\n"
+    "case IMPLICATION_MATCH:\n"
+    "delete implication_.precondition;\n"
+    "delete implication_.implied_template;\n"
+    "break;\n"
+    "case DYNAMIC_MATCH:\n"
+    "dyn_match->ref_count--;\n"
+    "if (dyn_match->ref_count == 0) {\n"
+    "delete dyn_match->ptr;\n"
+    "delete dyn_match;\n"
+    "}\n"
+    "break;\n"
+    "default:\n"
+    "break;\n"
+    "}\n"
     "template_selection = UNINITIALIZED_TEMPLATE;\n"
     "}\n\n", name);
 
@@ -1249,6 +1304,17 @@ void defEnumTemplate(const enum_def *edef, output_struct *output)
     "if (value_list.list_value[list_count].match(other_value)) "
     "return template_selection == VALUE_LIST;\n"
     "return template_selection == COMPLEMENTED_LIST;\n"
+    "case CONJUNCTION_MATCH:\n"
+    "for (unsigned int i = 0; i < value_list.n_values; i++) {\n"
+    "if (!value_list.list_value[i].match(other_value)) {\n"
+    "return FALSE;\n"
+    "}\n"
+    "}\n"
+    "return TRUE;\n"
+    "case IMPLICATION_MATCH:\n"
+    "return !implication_.precondition->match(other_value) || implication_.implied_template->match(other_value);\n"
+    "case DYNAMIC_MATCH:\n"
+    "return dyn_match->ptr->match(other_value);\n"
     "default:\n"
     "TTCN_error(\"Matching an uninitialized/unsupported template of "
     "enumerated type %s.\");\n"
@@ -1286,7 +1352,8 @@ void defEnumTemplate(const enum_def *edef, output_struct *output)
     "void %s_template::set_type(template_sel template_type, "
     "unsigned int list_length)\n"
     "{\n"
-    "if (template_type != VALUE_LIST && template_type != COMPLEMENTED_LIST) "
+    "if (template_type != VALUE_LIST && template_type != COMPLEMENTED_LIST && "
+    "template_type != CONJUNCTION_MATCH) "
     "TTCN_error(\"Setting an invalid list type for a template of enumerated "
     "type %s.\");\n"
     "clean_up();\n"
@@ -1301,7 +1368,8 @@ void defEnumTemplate(const enum_def *edef, output_struct *output)
     "%s_template& %s_template::list_item(unsigned int list_index)\n"
     "{\n"
     "if (template_selection != VALUE_LIST && "
-    "template_selection != COMPLEMENTED_LIST) "
+    "template_selection != COMPLEMENTED_LIST && "
+    "template_selection != CONJUNCTION_MATCH) "
     "TTCN_error(\"Accessing a list element in a non-list template of "
     "enumerated type %s.\");\n"
     "if (list_index >= value_list.n_values) "
@@ -1358,6 +1426,10 @@ void defEnumTemplate(const enum_def *edef, output_struct *output)
      "break;\n"
      "case COMPLEMENTED_LIST:\n"
      "TTCN_Logger::log_event_str(\"complement\");\n"
+     "case CONJUNCTION_MATCH:\n"
+     "if (template_selection == CONJUNCTION_MATCH) {\n"
+     "TTCN_Logger::log_event_str(\"conjunct\");\n"
+     "}\n"
      "case VALUE_LIST:\n"
      "TTCN_Logger::log_char('(');\n"
      "for (unsigned int elem_count = 0; elem_count < value_list.n_values; "
@@ -1367,6 +1439,14 @@ void defEnumTemplate(const enum_def *edef, output_struct *output)
      "}\n"
      "TTCN_Logger::log_char(')');\n"
      "break;\n"
+     "case IMPLICATION_MATCH:\n"
+     "implication_.precondition->log();\n"
+     "TTCN_Logger::log_event_str(\" implies \");\n"
+     "implication_.implied_template->log();\n"
+     "break;\n"
+     "case DYNAMIC_MATCH:\n"
+     "TTCN_Logger::log_event_str(\"@dynamic template\");\n"
+     "break;\n"
      "default:\n"
      "log_generic();\n"
      "}\n"
@@ -1461,6 +1541,8 @@ void defEnumTemplate(const enum_def *edef, output_struct *output)
     "case OMIT_VALUE:\n"
     "case ANY_OR_OMIT:\n"
     "return TRUE;\n"
+    "case IMPLICATION_MATCH:\n"
+    "return !implication_.precondition->match_omit() || implication_.implied_template->match_omit();\n"
     "case VALUE_LIST:\n"
     "case COMPLEMENTED_LIST:\n"
     "if (legacy) {\n"
@@ -1511,10 +1593,12 @@ void defEnumTemplate(const enum_def *edef, output_struct *output)
     "    *this = ANY_OR_OMIT;\n"
     "    break;\n"
     "  case Module_Param::MP_List_Template:\n"
-    "  case Module_Param::MP_ComplementList_Template: {\n"
+    "  case Module_Param::MP_ComplementList_Template:\n"
+    "  case Module_Param::MP_ConjunctList_Template: {\n"
     "    %s_template new_temp;\n"
     "    new_temp.set_type(m_p->get_type()==Module_Param::MP_List_Template ? "
-    "VALUE_LIST : COMPLEMENTED_LIST, m_p->get_size());\n"
+    "VALUE_LIST : (m_p->get_type() == Module_Param::MP_ConjunctList_Template ? "
+    "CONJUNCTION_MATCH : COMPLEMENTED_LIST), m_p->get_size());\n"
     "    for (size_t p_i=0; p_i<m_p->get_size(); p_i++) {\n"
     "      new_temp.list_item(p_i).set_param(*m_p->get_elem(p_i));\n"
     "    }\n"
@@ -1527,11 +1611,19 @@ void defEnumTemplate(const enum_def *edef, output_struct *output)
     "    }\n"
     "    *this = enum_val;\n"
     "  } break;\n"
+    "  case Module_Param::MP_Implication_Template: {\n"
+    "    %s_template* precondition = new %s_template;\n"
+    "    precondition->set_param(*m_p->get_elem(0));\n"
+    "    %s_template* implied_template = new %s_template;\n"
+    "    implied_template->set_param(*m_p->get_elem(1));\n"
+    "    *this = %s_template(precondition, implied_template);\n"
+    "  } break;\n"
     "  default:\n"
     "    param.type_error(\"enumerated template\", \"%s\");\n"
     "  }\n"
     "  is_ifpresent = param.get_ifpresent()%s;\n"
-    "}\n\n", name, enum_type, name, name, dispname, dispname,
+    "}\n\n", name, enum_type, name, name, dispname,
+    name, name, name, name, name, dispname,
     use_runtime_2 ? " || m_p->get_ifpresent()" : "");
 
   /* get_param(), RT2 only */
@@ -1559,10 +1651,14 @@ void defEnumTemplate(const enum_def *edef, output_struct *output)
       "    m_p = new Module_Param_Enumerated(mcopystr(%s::enum_to_str(single_value)));\n"
       "    break;\n"
       "  case VALUE_LIST:\n"
-      "  case COMPLEMENTED_LIST: {\n"
+      "  case COMPLEMENTED_LIST:\n"
+      "  case CONJUNCTION_MATCH: {\n"
       "    if (template_selection == VALUE_LIST) {\n"
       "      m_p = new Module_Param_List_Template();\n"
       "    }\n"
+      "    else if (template_selection == CONJUNCTION_MATCH) {\n"
+      "      m_p = new Module_Param_ConjunctList_Template();\n"
+      "    }\n"
       "    else {\n"
       "      m_p = new Module_Param_ComplementList_Template();\n"
       "    }\n"
@@ -1570,6 +1666,11 @@ void defEnumTemplate(const enum_def *edef, output_struct *output)
       "      m_p->add_elem(value_list.list_value[i_i].get_param(param_name));\n"
       "    }\n"
       "    break; }\n"
+      "  case IMPLICATION_MATCH:\n"
+      "    m_p = new Module_Param_Implication_Template();\n"
+      "    m_p->add_elem(implication_.precondition->get_param(param_name));\n"
+      "    m_p->add_elem(implication_.implied_template->get_param(param_name));\n"
+      "    break;\n"
       "  default:\n"
       "    break;\n"
       "  }\n"
