@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2020 Ericsson Telecom AB
+ * Copyright (c) 2000-2021 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v2.0
  * which accompanies this distribution, and is available at
@@ -972,12 +972,15 @@ int main(int argc, char *argv[])
 
   if (vflag) {
     fputs("TTCN-3 and ASN.1 Compiler for the TTCN-3 Test Executor\n"
-	  "Product number: " PRODUCT_NUMBER "\n"
 	  "Version: " VERSION_STRING "\n"
 	  "Build date: " __DATE__ " " __TIME__ "\n"
 	  "Compiled with: " C_COMPILER_VERSION "\n", stderr);
 	  fputs("Using ", stderr);
 	  fputs(openssl_version_str(), stderr);
+	if (strlen(GIT_COMMIT_ID)) {
+	  fputs("\nCommit id: ", stderr);
+	  fputs(GIT_COMMIT_ID, stderr);
+	}
 	  fputs("\n\n" COPYRIGHT_STRING "\n\n", stderr);
 #ifdef LICENSE
     print_license_info();
