@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2020 Ericsson Telecom AB
+ * Copyright (c) 2000-2021 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v2.0
  * which accompanies this distribution, and is available at
@@ -644,7 +644,7 @@ void BITSTRING::decode_text(Text_Buf& text_buf)
 }
 
 void BITSTRING::encode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
-                       TTCN_EncDec::coding_t p_coding, ...) const
+                       int p_coding, ...) const
 {
   va_list pvar;
   va_start(pvar, p_coding);
@@ -697,7 +697,7 @@ void BITSTRING::encode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
 }
 
 void BITSTRING::decode(const TTCN_Typedescriptor_t& p_td, TTCN_Buffer& p_buf,
-                       TTCN_EncDec::coding_t p_coding, ...)
+                       int p_coding, ...)
 {
   va_list pvar;
   va_start(pvar, p_coding);
@@ -1204,11 +1204,16 @@ int BITSTRING::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer& p_
   size_t value_len = 0;
   boolean error = FALSE;
   size_t dec_len = 0;
-  boolean use_default = p_td.json->default_value && 0 == p_tok.get_buffer_length();
-  if (use_default) {
+  boolean use_default = FALSE;
+  if (p_td.json->default_value.type == JD_STANDARD && 0 == p_tok.get_buffer_length()) {
+    *this = *static_cast<const BITSTRING*>(p_td.json->default_value.val);
+    return dec_len;
+  }
+  if (p_td.json->default_value.type == JD_LEGACY && 0 == p_tok.get_buffer_length()) {
     // No JSON data in the buffer -> use default value
-    value = const_cast<char*>(p_td.json->default_value);
+    value = const_cast<char*>(p_td.json->default_value.str);
     value_len = strlen(value);
+    use_default = TRUE;
   } else {
     dec_len = p_tok.get_next_token(&token, &value, &value_len);
   }
@@ -1505,6 +1510,7 @@ void BITSTRING_template::clean_up()
   switch (template_selection) {
   case VALUE_LIST:
   case COMPLEMENTED_LIST:
+  case CONJUNCTION_MATCH:
     delete [] value_list.list_value;
     break;
   case STRING_PATTERN:
@@ -1526,6 +1532,17 @@ void BITSTRING_template::clean_up()
         "decoded content match.");
     }
     break;
+  case IMPLICATION_MATCH:
+    delete implication_.precondition;
+    delete implication_.implied_template;
+    break;
+  case DYNAMIC_MATCH:
+    dyn_match->ref_count--;
+    if (dyn_match->ref_count == 0) {
+      delete dyn_match->ptr;
+      delete dyn_match;
+    }
+    break;
   default:
     break;
   }
@@ -1544,6 +1561,7 @@ void BITSTRING_template::copy_template(const BITSTRING_template& other_value)
     break;
   case VALUE_LIST:
   case COMPLEMENTED_LIST:
+  case CONJUNCTION_MATCH:
     value_list.n_values = other_value.value_list.n_values;
     value_list.list_value = new BITSTRING_template[value_list.n_values];
     for (unsigned int i = 0; i < value_list.n_values; i++)
@@ -1558,6 +1576,14 @@ void BITSTRING_template::copy_template(const BITSTRING_template& other_value)
     dec_match = other_value.dec_match;
     dec_match->ref_count++;
     break;
+  case IMPLICATION_MATCH:
+    implication_.precondition = new BITSTRING_template(*other_value.implication_.precondition);
+    implication_.implied_template = new BITSTRING_template(*other_value.implication_.implied_template);
+    break;
+  case DYNAMIC_MATCH:
+    dyn_match = other_value.dyn_match;
+    dyn_match->ref_count++;
+    break;
   default:
     TTCN_error("Copying an uninitialized/unsupported bitstring template.");
   }
@@ -1704,6 +1730,21 @@ BITSTRING_template::BITSTRING_template(const BITSTRING_template& other_value)
   copy_template(other_value);
 }
 
+BITSTRING_template::BITSTRING_template(BITSTRING_template* p_precondition, BITSTRING_template* p_implied_template)
+: Restricted_Length_Template(IMPLICATION_MATCH)
+{
+  implication_.precondition = p_precondition;
+  implication_.implied_template = p_implied_template;
+}
+
+BITSTRING_template::BITSTRING_template(Dynamic_Match_Interface<BITSTRING>* p_dyn_match)
+: Restricted_Length_Template(DYNAMIC_MATCH)
+{
+  dyn_match = new dynmatch_struct<BITSTRING>;
+  dyn_match->ptr = p_dyn_match;
+  dyn_match->ref_count = 1;
+}
+
 BITSTRING_template& BITSTRING_template::operator=(template_sel other_value)
 {
   check_single_selection(other_value);
@@ -2029,6 +2070,17 @@ boolean BITSTRING_template::match(const BITSTRING& other_value,
     TTCN_EncDec::set_error_behavior(TTCN_EncDec::ET_ALL,TTCN_EncDec::EB_DEFAULT);
     TTCN_EncDec::clear_error();
     return ret_val; }
+  case CONJUNCTION_MATCH:
+    for (unsigned int i = 0; i < value_list.n_values; i++) {
+      if (!value_list.list_value[i].match(other_value)) {
+        return FALSE;
+      }
+    }
+    return TRUE;
+  case IMPLICATION_MATCH:
+    return !implication_.precondition->match(other_value) || implication_.implied_template->match(other_value);
+  case DYNAMIC_MATCH:
+    return dyn_match->ptr->match(other_value);
   default:
     TTCN_error("Matching an uninitialized/unsupported bitstring template.");
   }
@@ -2084,6 +2136,15 @@ int BITSTRING_template::lengthof() const
   case COMPLEMENTED_LIST:
     TTCN_error("Performing lengthof() operation on a bitstring template "
                "containing complemented list.");
+  case CONJUNCTION_MATCH:
+    TTCN_error("Performing lengthof() operation on a bitstring template "
+               "containing a conjunction list match.");
+  case IMPLICATION_MATCH:
+    TTCN_error("Performing lengthof() operation on a bitstring template "
+               "containing an implication match.");
+  case DYNAMIC_MATCH:
+    TTCN_error("Performing lengthof() operation on a bitstring template "
+               "containing a dynamic match.");
   case STRING_PATTERN:
     min_length = 0;
     has_any_or_none = FALSE; // TRUE if * chars in the pattern
@@ -2105,7 +2166,7 @@ void BITSTRING_template::set_type(template_sel template_type,
   unsigned int list_length /* = 0 */)
 {
   if (template_type != VALUE_LIST && template_type != COMPLEMENTED_LIST &&
-      template_type != DECODE_MATCH)
+      template_type != DECODE_MATCH && template_type != CONJUNCTION_MATCH)
     TTCN_error("Setting an invalid list type for a bitstring template.");
   clean_up();
   set_selection(template_type);
@@ -2118,7 +2179,8 @@ void BITSTRING_template::set_type(template_sel template_type,
 BITSTRING_template& BITSTRING_template::list_item(unsigned int list_index)
 {
   if (template_selection != VALUE_LIST &&
-      template_selection != COMPLEMENTED_LIST)
+      template_selection != COMPLEMENTED_LIST &&
+      template_selection != CONJUNCTION_MATCH)
     TTCN_error("Accessing a list element of a non-list bitstring template.");
   if (list_index >= value_list.n_values)
     TTCN_error("Index overflow in a bitstring value list template.");
@@ -2165,6 +2227,11 @@ void BITSTRING_template::log() const
   case COMPLEMENTED_LIST:
     TTCN_Logger::log_event_str("complement");
     // no break
+  case CONJUNCTION_MATCH:
+    if (template_selection == CONJUNCTION_MATCH) {
+      TTCN_Logger::log_event_str("conjunct");
+    }
+    // no break
   case VALUE_LIST:
     TTCN_Logger::log_char('(');
     for (unsigned int i = 0; i < value_list.n_values; i++) {
@@ -2186,6 +2253,14 @@ void BITSTRING_template::log() const
     TTCN_Logger::log_event_str("decmatch ");
     dec_match->instance->log();
     break;
+  case IMPLICATION_MATCH:
+    implication_.precondition->log();
+    TTCN_Logger::log_event_str(" implies ");
+    implication_.implied_template->log();
+    break;
+  case DYNAMIC_MATCH:
+    TTCN_Logger::log_event_str("@dynamic template");
+    break;
   default:
     log_generic();
     break;
@@ -2228,10 +2303,12 @@ void BITSTRING_template::set_param(Module_Param& param) {
     *this = ANY_OR_OMIT;
     break;
   case Module_Param::MP_List_Template:
-  case Module_Param::MP_ComplementList_Template: {
+  case Module_Param::MP_ComplementList_Template:
+  case Module_Param::MP_ConjunctList_Template:  {
     BITSTRING_template temp;
     temp.set_type(mp->get_type() == Module_Param::MP_List_Template ?
-      VALUE_LIST : COMPLEMENTED_LIST, mp->get_size());
+      VALUE_LIST : (mp->get_type() == Module_Param::MP_ConjunctList_Template ?
+      CONJUNCTION_MATCH : COMPLEMENTED_LIST), mp->get_size());
     for (size_t i=0; i<mp->get_size(); i++) {
       temp.list_item(i).set_param(*mp->get_elem(i));
     }
@@ -2243,6 +2320,13 @@ void BITSTRING_template::set_param(Module_Param& param) {
   case Module_Param::MP_Bitstring_Template:
     *this = BITSTRING_template(mp->get_string_size(), (unsigned char*)mp->get_string_data());
     break;
+  case Module_Param::MP_Implication_Template: {
+    BITSTRING_template* precondition = new BITSTRING_template;
+    precondition->set_param(*mp->get_elem(0));
+    BITSTRING_template* implied_template = new BITSTRING_template;
+    implied_template->set_param(*mp->get_elem(1));
+    *this = BITSTRING_template(precondition, implied_template);
+  } break;
   case Module_Param::MP_Expression:
     if (mp->get_expr_type() == Module_Param::EXPR_CONCATENATE) {
       BITSTRING operand1, operand2;
@@ -2287,10 +2371,14 @@ Module_Param* BITSTRING_template::get_param(Module_Param_Name& param_name) const
     mp = single_value.get_param(param_name);
     break;
   case VALUE_LIST:
-  case COMPLEMENTED_LIST: {
+  case COMPLEMENTED_LIST:
+  case CONJUNCTION_MATCH: {
     if (template_selection == VALUE_LIST) {
       mp = new Module_Param_List_Template();
     }
+    else if (template_selection == CONJUNCTION_MATCH) {
+      mp = new Module_Param_ConjunctList_Template();
+    }
     else {
       mp = new Module_Param_ComplementList_Template();
     }
@@ -2303,6 +2391,11 @@ Module_Param* BITSTRING_template::get_param(Module_Param_Name& param_name) const
     memcpy(val_cpy, pattern_value->elements_ptr, pattern_value->n_elements);
     mp = new Module_Param_Bitstring_Template(pattern_value->n_elements, val_cpy);
     break; }
+  case IMPLICATION_MATCH:
+    mp = new Module_Param_Implication_Template();
+    mp->add_elem(implication_.precondition->get_param(param_name));
+    mp->add_elem(implication_.implied_template->get_param(param_name));
+    break;
   case DECODE_MATCH:
     TTCN_error("Referencing a decoded content matching template is not supported.");
     break;
@@ -2391,6 +2484,8 @@ boolean BITSTRING_template::match_omit(boolean legacy /* = FALSE */) const
   case OMIT_VALUE:
   case ANY_OR_OMIT:
     return TRUE;
+  case IMPLICATION_MATCH:
+    return !implication_.precondition->match_omit() || implication_.implied_template->match_omit();
   case VALUE_LIST:
   case COMPLEMENTED_LIST:
     if (legacy) {
