@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (c) 2000-2020 Ericsson Telecom AB
+ * Copyright (c) 2000-2021 Ericsson Telecom AB
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v2.0
  * which accompanies this distribution, and is available at
@@ -598,7 +598,7 @@ void OCTETSTRING::decode_text(Text_Buf& text_buf)
 
 void OCTETSTRING::encode(const TTCN_Typedescriptor_t& p_td,
                          TTCN_Buffer& p_buf,
-                         TTCN_EncDec::coding_t p_coding, ...) const
+                         int p_coding, ...) const
 {
   va_list pvar;
   va_start(pvar, p_coding);
@@ -659,7 +659,7 @@ void OCTETSTRING::encode(const TTCN_Typedescriptor_t& p_td,
 
 void OCTETSTRING::decode(const TTCN_Typedescriptor_t& p_td,
                          TTCN_Buffer& p_buf,
-                         TTCN_EncDec::coding_t p_coding, ...)
+                         int p_coding, ...)
 {
   va_list pvar;
   va_start(pvar, p_coding);
@@ -1336,11 +1336,16 @@ int OCTETSTRING::JSON_decode(const TTCN_Typedescriptor_t& p_td, JSON_Tokenizer&
   size_t value_len = 0;
   boolean error = FALSE;
   size_t dec_len = 0;
-  boolean use_default = p_td.json->default_value && 0 == p_tok.get_buffer_length();
-  if (use_default) {
+  boolean use_default = FALSE;
+  if (p_td.json->default_value.type == JD_STANDARD && 0 == p_tok.get_buffer_length()) {
+    *this = *static_cast<const OCTETSTRING*>(p_td.json->default_value.val);
+    return dec_len;
+  }
+  if (p_td.json->default_value.type == JD_LEGACY && 0 == p_tok.get_buffer_length()) {
     // No JSON data in the buffer -> use default value
-    value = const_cast<char*>(p_td.json->default_value);
+    value = const_cast<char*>(p_td.json->default_value.str);
     value_len = strlen(value);
+    use_default = TRUE;
   } else {
     dec_len = p_tok.get_next_token(&token, &value, &value_len);
   }
@@ -1629,6 +1634,7 @@ void OCTETSTRING_template::clean_up()
   switch (template_selection) {
   case VALUE_LIST:
   case COMPLEMENTED_LIST:
+  case CONJUNCTION_MATCH:
     delete [] value_list.list_value;
     break;
   case STRING_PATTERN:
@@ -1650,6 +1656,17 @@ void OCTETSTRING_template::clean_up()
         "decoded content match.");
     }
     break;
+  case IMPLICATION_MATCH:
+    delete implication_.precondition;
+    delete implication_.implied_template;
+    break;
+  case DYNAMIC_MATCH:
+    dyn_match->ref_count--;
+    if (dyn_match->ref_count == 0) {
+      delete dyn_match->ptr;
+      delete dyn_match;
+    }
+    break;
   default:
     break;
   }
@@ -1669,6 +1686,7 @@ void OCTETSTRING_template::copy_template
     break;
   case VALUE_LIST:
   case COMPLEMENTED_LIST:
+  case CONJUNCTION_MATCH:
     value_list.n_values = other_value.value_list.n_values;
     value_list.list_value = new OCTETSTRING_template[value_list.n_values];
     for (unsigned int i = 0; i < value_list.n_values; i++)
@@ -1683,6 +1701,14 @@ void OCTETSTRING_template::copy_template
     dec_match = other_value.dec_match;
     dec_match->ref_count++;
     break;
+  case IMPLICATION_MATCH:
+    implication_.precondition = new OCTETSTRING_template(*other_value.implication_.precondition);
+    implication_.implied_template = new OCTETSTRING_template(*other_value.implication_.implied_template);
+    break;
+  case DYNAMIC_MATCH:
+    dyn_match = other_value.dyn_match;
+    dyn_match->ref_count++;
+    break;
   default:
     TTCN_error("Copying an uninitialized/unsupported octetstring template.");
   }
@@ -1818,6 +1844,21 @@ OCTETSTRING_template::OCTETSTRING_template
   copy_template(other_value);
 }
 
+OCTETSTRING_template::OCTETSTRING_template(OCTETSTRING_template* p_precondition, OCTETSTRING_template* p_implied_template)
+: Restricted_Length_Template(IMPLICATION_MATCH)
+{
+  implication_.precondition = p_precondition;
+  implication_.implied_template = p_implied_template;
+}
+
+OCTETSTRING_template::OCTETSTRING_template(Dynamic_Match_Interface<OCTETSTRING>* p_dyn_match)
+: Restricted_Length_Template(DYNAMIC_MATCH)
+{
+  dyn_match = new dynmatch_struct<OCTETSTRING>;
+  dyn_match->ptr = p_dyn_match;
+  dyn_match->ref_count = 1;
+}
+
 OCTETSTRING_template::~OCTETSTRING_template()
 {
   clean_up();
@@ -2148,6 +2189,17 @@ boolean OCTETSTRING_template::match(const OCTETSTRING& other_value,
     TTCN_EncDec::set_error_behavior(TTCN_EncDec::ET_ALL,TTCN_EncDec::EB_DEFAULT);
     TTCN_EncDec::clear_error();
     return ret_val; }
+  case CONJUNCTION_MATCH:
+    for (unsigned int i = 0; i < value_list.n_values; i++) {
+      if (!value_list.list_value[i].match(other_value)) {
+        return FALSE;
+      }
+    }
+    return TRUE;
+  case IMPLICATION_MATCH:
+    return !implication_.precondition->match(other_value) || implication_.implied_template->match(other_value);
+  case DYNAMIC_MATCH:
+    return dyn_match->ptr->match(other_value);
   default:
     TTCN_error("Matching an uninitialized/unsupported octetstring template.");
   }
@@ -2212,6 +2264,15 @@ int OCTETSTRING_template::lengthof() const
       else has_any_or_none = TRUE;   // case of * character
     }
     break;
+  case CONJUNCTION_MATCH:
+    TTCN_error("Performing lengthof() operation on a octetstring template "
+               "containing a conjunction list match.");
+  case IMPLICATION_MATCH:
+    TTCN_error("Performing lengthof() operation on a octetstring template "
+               "containing an implication match.");
+  case DYNAMIC_MATCH:
+    TTCN_error("Performing lengthof() operation on a octetstring template "
+               "containing a dynamic match.");
   default:
     TTCN_error("Performing lengthof() operation on an "
                "uninitialized/unsupported octetstring template.");
@@ -2224,7 +2285,7 @@ void OCTETSTRING_template::set_type(template_sel template_type,
   unsigned int list_length)
 {
   if (template_type != VALUE_LIST && template_type != COMPLEMENTED_LIST &&
-      template_type != DECODE_MATCH)
+      template_type != DECODE_MATCH && template_type != CONJUNCTION_MATCH)
     TTCN_error("Setting an invalid type for an octetstring template.");
   clean_up();
   set_selection(template_type);
@@ -2237,7 +2298,8 @@ void OCTETSTRING_template::set_type(template_sel template_type,
 OCTETSTRING_template& OCTETSTRING_template::list_item(unsigned int list_index)
 {
   if (template_selection != VALUE_LIST &&
-      template_selection != COMPLEMENTED_LIST)
+      template_selection != COMPLEMENTED_LIST &&
+      template_selection != CONJUNCTION_MATCH)
     TTCN_error("Accessing a list element of a non-list octetstring template.");
   if (list_index >= value_list.n_values)
     TTCN_error("Index overflow in an octetstring value list template.");
@@ -2282,6 +2344,11 @@ void OCTETSTRING_template::log() const
   case COMPLEMENTED_LIST:
     TTCN_Logger::log_event_str("complement");
     // no break
+  case CONJUNCTION_MATCH:
+    if (template_selection == CONJUNCTION_MATCH) {
+      TTCN_Logger::log_event_str("conjunct");
+    }
+    // no break
   case VALUE_LIST:
     TTCN_Logger::log_char('(');
     for(unsigned int i = 0; i < value_list.n_values; i++) {
@@ -2305,6 +2372,14 @@ void OCTETSTRING_template::log() const
     TTCN_Logger::log_event_str("decmatch ");
     dec_match->instance->log();
     break;
+  case IMPLICATION_MATCH:
+    implication_.precondition->log();
+    TTCN_Logger::log_event_str(" implies ");
+    implication_.implied_template->log();
+    break;
+  case DYNAMIC_MATCH:
+    TTCN_Logger::log_event_str("@dynamic template");
+    break;
   default:
     log_generic();
     break;
@@ -2347,10 +2422,12 @@ void OCTETSTRING_template::set_param(Module_Param& param) {
     *this = ANY_OR_OMIT;
     break;
   case Module_Param::MP_List_Template:
-  case Module_Param::MP_ComplementList_Template: {
+  case Module_Param::MP_ComplementList_Template:
+  case Module_Param::MP_ConjunctList_Template: {
     OCTETSTRING_template temp;
     temp.set_type(mp->get_type() == Module_Param::MP_List_Template ?
-      VALUE_LIST : COMPLEMENTED_LIST, mp->get_size());
+      VALUE_LIST : (mp->get_type() == Module_Param::MP_ConjunctList_Template ?
+      CONJUNCTION_MATCH : COMPLEMENTED_LIST), mp->get_size());
     for (size_t i=0; i<mp->get_size(); i++) {
       temp.list_item(i).set_param(*mp->get_elem(i));
     }
@@ -2362,6 +2439,13 @@ void OCTETSTRING_template::set_param(Module_Param& param) {
   case Module_Param::MP_Octetstring_Template:
     *this = OCTETSTRING_template(mp->get_string_size(), (unsigned short*)mp->get_string_data());
     break;
+  case Module_Param::MP_Implication_Template: {
+    OCTETSTRING_template* precondition = new OCTETSTRING_template;
+    precondition->set_param(*mp->get_elem(0));
+    OCTETSTRING_template* implied_template = new OCTETSTRING_template;
+    implied_template->set_param(*mp->get_elem(1));
+    *this = OCTETSTRING_template(precondition, implied_template);
+  } break;
   case Module_Param::MP_Expression:
     if (mp->get_expr_type() == Module_Param::EXPR_CONCATENATE) {
       OCTETSTRING operand1, operand2;
@@ -2406,10 +2490,14 @@ Module_Param* OCTETSTRING_template::get_param(Module_Param_Name& param_name) con
     mp = single_value.get_param(param_name);
     break;
   case VALUE_LIST:
-  case COMPLEMENTED_LIST: {
+  case COMPLEMENTED_LIST:
+  case CONJUNCTION_MATCH: {
     if (template_selection == VALUE_LIST) {
       mp = new Module_Param_List_Template();
     }
+    else if (template_selection == CONJUNCTION_MATCH) {
+      mp = new Module_Param_ConjunctList_Template();
+    }
     else {
       mp = new Module_Param_ComplementList_Template();
     }
@@ -2424,6 +2512,11 @@ Module_Param* OCTETSTRING_template::get_param(Module_Param_Name& param_name) con
       sizeof(unsigned short));
     mp = new Module_Param_Octetstring_Template(pattern_value->n_elements, val_cpy);
     break; }
+  case IMPLICATION_MATCH:
+    mp = new Module_Param_Implication_Template();
+    mp->add_elem(implication_.precondition->get_param(param_name));
+    mp->add_elem(implication_.implied_template->get_param(param_name));
+    break;
   case DECODE_MATCH:
     TTCN_error("Referencing a decoded content matching template is not supported.");
     break;
@@ -2515,6 +2608,8 @@ boolean OCTETSTRING_template::match_omit(boolean legacy /* = FALSE */) const
   case OMIT_VALUE:
   case ANY_OR_OMIT:
     return TRUE;
+  case IMPLICATION_MATCH:
+    return !implication_.precondition->match_omit() || implication_.implied_template->match_omit();
   case VALUE_LIST:
   case COMPLEMENTED_LIST:
     if (legacy) {
